@@ -154,36 +154,55 @@ make_view_config(std::shared_ptr<t_schema> schema, t_val date_parser, t_val conf
          * as all lookups into `schema` must be valid for all computed
          * columns on the View.
          */
-        std::vector<t_dtype> input_types;
-        for (const auto& input_column : input_columns) {
-            input_types.push_back(schema->get_dtype(input_column));
+        std::vector<t_dtype> input_types(input_columns.size());
+
+        // If the input columns are invalid, an error will be thrown here.
+        for (auto i = 0; i < input_columns.size(); ++i) {
+            input_types[i] = schema->get_dtype(input_columns[i]);
         }
 
         t_computation computation = t_computed_column::get_computation(
             computed_function_name, input_types);
         
+        // Throw an exception if the computation is invalid - the UI will
+        // prevent users from saving invalidly-typed computations.
         if (computation.m_name == INVALID_COMPUTED_FUNCTION) {
-            std::cerr 
-                << "Could not build computed column definition for `" 
-                << computed_column_name 
-                << "`" 
-                << std::endl;
-            continue;
+            // Get actual input types
+            auto valid_input_types = t_computed_column::get_computation_input_types(computed_function_name);
+            std::stringstream ss;
+            ss << "View creation failed: could not build computed column '"
+               << computed_column_name
+               << "' as the input column types are invalid."
+               << std::endl;
+            PSP_COMPLAIN_AND_ABORT(ss.str());
         }
 
         t_dtype output_column_type = computation.m_return_type;
 
-        // Add the column to the schema.
-        schema->add_column(computed_column_name, output_column_type);
+        // If the computed column does not overwrite a "real" column, add it
+        // to the schema, otherwise throw an exception. The viewer UI will
+        // prevent users from overwriting "real" columns, so the only case
+        // here is when views are created programatically, where we should
+        // fail fast and report the error cleanly.
+        if (!schema->has_column(computed_column_name)) {
+            schema->add_column(computed_column_name, output_column_type);
 
-        // Add the computed column to the config.
-        auto tp = std::make_tuple(
-            computed_column_name,
-            computed_function_name,
-            input_columns,
-            computation);
+            // Add the computed column to the config.
+            auto tp = std::make_tuple(
+                computed_column_name,
+                computed_function_name,
+                input_columns,
+                computation);
 
-        computed_columns.push_back(tp);
+            computed_columns.push_back(tp);
+        } else {
+            std::stringstream ss;
+            ss << "View creation failed: cannot overwrite Table column '"
+               << computed_column_name
+               << "' with a computed column."
+               << std::endl;
+            PSP_COMPLAIN_AND_ABORT(ss.str());
+        }        
     }
 
     // construct filters with filter terms, and fill the vector of tuples
@@ -236,16 +255,29 @@ make_view_config(std::shared_ptr<t_schema> schema, t_val date_parser, t_val conf
     return view_config;
 }
 
+/******************************************************************************
+ *
+ * make_view
+ */
+
 template <typename CTX_T>
 std::shared_ptr<View<CTX_T>>
 make_view(std::shared_ptr<Table> table, const std::string& name, const std::string& separator,
     t_val view_config, t_val date_parser) {
     std::shared_ptr<t_schema> schema = std::make_shared<t_schema>(table->get_schema());
-    std::shared_ptr<t_view_config> config = 
-        make_view_config<t_val>(schema, date_parser, view_config);
-    auto ctx = make_context<CTX_T>(table, schema, config, name);
-    auto view_ptr = std::make_shared<View<CTX_T>>(table, ctx, name, separator, config);
-    return view_ptr;
+    std::shared_ptr<t_view_config> config = make_view_config<t_val>(schema, date_parser, view_config);
+    {
+        PerspectiveScopedGILRelease acquire(table->get_pool()->get_event_loop_thread_id());
+        auto ctx = make_context<CTX_T>(table, schema, config, name);
+        auto view_ptr = std::make_shared<View<CTX_T>>(table, ctx, name, separator, config);
+        return view_ptr;
+    }
+}
+
+std::shared_ptr<View<t_ctxunit>>
+make_view_unit(std::shared_ptr<Table> table, std::string name, std::string separator,
+    t_val view_config, t_val date_parser) {
+    return make_view<t_ctxunit>(table, name, separator, view_config, date_parser);
 }
 
 std::shared_ptr<View<t_ctx0>>
@@ -266,6 +298,25 @@ make_view_ctx2(std::shared_ptr<Table> table, std::string name, std::string separ
     return make_view<t_ctx2>(table, name, separator, view_config, date_parser);
 }
 
+/******************************************************************************
+ *
+ * to_arrow
+ */
+
+py::bytes
+to_arrow_unit(
+    std::shared_ptr<View<t_ctxunit>> view,
+    std::int32_t start_row,
+    std::int32_t end_row,
+    std::int32_t start_col,
+    std::int32_t end_col
+) {
+    PerspectiveScopedGILRelease acquire(view->get_event_loop_thread_id());
+    std::shared_ptr<std::string> str = 
+        view->to_arrow(start_row, end_row, start_col, end_col);
+    return py::bytes(*str);
+}
+
 py::bytes
 to_arrow_zero(
     std::shared_ptr<View<t_ctx0>> view,
@@ -274,6 +325,7 @@ to_arrow_zero(
     std::int32_t start_col,
     std::int32_t end_col
 ) {
+    PerspectiveScopedGILRelease acquire(view->get_event_loop_thread_id());
     std::shared_ptr<std::string> str = 
         view->to_arrow(start_row, end_row, start_col, end_col);
     return py::bytes(*str);
@@ -287,6 +339,7 @@ to_arrow_one(
     std::int32_t start_col, 
     std::int32_t end_col
 ) {
+    PerspectiveScopedGILRelease acquire(view->get_event_loop_thread_id());
     std::shared_ptr<std::string> str = 
         view->to_arrow(start_row, end_row, start_col, end_col);
     return py::bytes(*str);
@@ -300,13 +353,28 @@ to_arrow_two(
     std::int32_t start_col, 
     std::int32_t end_col
 ) {
+    PerspectiveScopedGILRelease acquire(view->get_event_loop_thread_id());
     std::shared_ptr<std::string> str = 
         view->to_arrow(start_row, end_row, start_col, end_col);
     return py::bytes(*str);
 }
 
+/******************************************************************************
+ *
+ * get_row_delta
+ */
+
+py::bytes
+get_row_delta_unit(std::shared_ptr<View<t_ctxunit>> view) {
+    PerspectiveScopedGILRelease acquire(view->get_event_loop_thread_id());
+    std::shared_ptr<t_data_slice<t_ctxunit>> slice = view->get_row_delta();
+    std::shared_ptr<std::string> arrow = view->data_slice_to_arrow(slice);
+    return py::bytes(*arrow);
+}
+
 py::bytes
 get_row_delta_zero(std::shared_ptr<View<t_ctx0>> view) {
+    PerspectiveScopedGILRelease acquire(view->get_event_loop_thread_id());
     std::shared_ptr<t_data_slice<t_ctx0>> slice = view->get_row_delta();
     std::shared_ptr<std::string> arrow = view->data_slice_to_arrow(slice);
     return py::bytes(*arrow);
@@ -314,6 +382,7 @@ get_row_delta_zero(std::shared_ptr<View<t_ctx0>> view) {
 
 py::bytes
 get_row_delta_one(std::shared_ptr<View<t_ctx1>> view) {
+    PerspectiveScopedGILRelease acquire(view->get_event_loop_thread_id());
     std::shared_ptr<t_data_slice<t_ctx1>> slice = view->get_row_delta();
     std::shared_ptr<std::string> arrow = view->data_slice_to_arrow(slice);
     return py::bytes(*arrow);
@@ -322,14 +391,12 @@ get_row_delta_one(std::shared_ptr<View<t_ctx1>> view) {
 py::bytes
 get_row_delta_two(
     std::shared_ptr<View<t_ctx2>> view) {
+    PerspectiveScopedGILRelease acquire(view->get_event_loop_thread_id());
     std::shared_ptr<t_data_slice<t_ctx2>> slice = view->get_row_delta();
     std::shared_ptr<std::string> arrow = view->data_slice_to_arrow(slice);
     return py::bytes(*arrow);
 }
 
-
-
-
 } //namespace binding
 } //namespace perspective
 
