@@ -8,43 +8,67 @@
 
 import six
 from random import random
-from .validate import validate_plugin, validate_columns, validate_row_pivots, \
-    validate_column_pivots, validate_aggregates, validate_sort, \
-    validate_filters, validate_computed_columns, \
-    validate_plugin_config
+from .validate import (
+    validate_plugin,
+    validate_columns,
+    validate_row_pivots,
+    validate_column_pivots,
+    validate_aggregates,
+    validate_sort,
+    validate_filters,
+    validate_computed_columns,
+    validate_plugin_config,
+)
 from .viewer_traitlets import PerspectiveTraitlets
 from ..libpsp import is_libpsp
 
 if is_libpsp():
-    from ..libpsp import Table, PerspectiveManager
+    from ..libpsp import Table, View, PerspectiveManager
+    from ..core.exception import PerspectiveError
 
 
 class PerspectiveViewer(PerspectiveTraitlets, object):
-    '''PerspectiveViewer wraps the `perspective.Table` API and exposes an API
+    """PerspectiveViewer wraps the `perspective.Table` API and exposes an API
     around creating views, loading data, and updating data.
-    '''
+    """
 
     # Keep track of attributes that can be set via Enum, and their
     # validation methods.
     ENUM_VALIDATORS = {
         "plugin": validate_plugin,
         "aggregates": validate_aggregates,
-        "sort": validate_sort
+        "sort": validate_sort,
     }
 
-    def __init__(self,
-                 plugin='datagrid',
-                 columns=None,
-                 row_pivots=None,
-                 column_pivots=None,
-                 aggregates=None,
-                 sort=None,
-                 filters=None,
-                 computed_columns=None,
-                 plugin_config=None,
-                 dark=None,
-                 editable=False):
-        '''Initialize an instance of `PerspectiveViewer` with the given viewer
+    # Viewer attributes that should be saved in `save()` and restored using
+    # `restore()`. Symmetric to `PERSISTENT_ATTRIBUTES` in `perspective-viewer`.
+    PERSISTENT_ATTRIBUTES = (
+        "row_pivots",
+        "column_pivots",
+        "filters",
+        "sort",
+        "aggregates",
+        "columns",
+        "computed_columns",
+        "plugin",
+        "editable",
+    )
+
+    def __init__(
+        self,
+        plugin="datagrid",
+        columns=None,
+        row_pivots=None,
+        column_pivots=None,
+        aggregates=None,
+        sort=None,
+        filters=None,
+        computed_columns=None,
+        plugin_config=None,
+        dark=None,
+        editable=False,
+    ):
+        """Initialize an instance of `PerspectiveViewer` with the given viewer
         configuration.  Do not pass a `Table` or data into the constructor -
         use the :func:`load()` method to provide the viewer with data.
 
@@ -69,11 +93,11 @@ class PerspectiveViewer(PerspectiveTraitlets, object):
                 each dict containing ``column``, a new computed column name,
                 ``computed_function_name``, a string computed function name,
                 and ``inputs``, a list of input column names.
-            plugin (:obj:`str`/:obj:`perspective.Plugin`): Which plugin to select by
-                default.
+            plugin (:obj:`str`/:obj:`perspective.Plugin`): Which plugin to
+                select by default.
             plugin_config (:obj:`dict`): Custom config for all plugins by name.
+            dark (:obj:`bool`): Whether to invert the color theme.
             editable (:obj:`bool`): Whether to allow editability using the grid.
-            dark (:obj:`bool`): Whether to invert the colors.
 
         Examples:
             >>> viewer = PerspectiveViewer(
@@ -87,14 +111,16 @@ class PerspectiveViewer(PerspectiveTraitlets, object):
             ...         "inputs": ["Sales", "Profit"]
             ...     }]
             ... )
-        '''
+        """
 
         # Create an instance of `PerspectiveManager`, which receives messages
         # from the `PerspectiveJupyterClient` on the front-end.
         if is_libpsp():
             self.manager = PerspectiveManager()
-        self.table_name = None  # not a traitlet - only used in python
-        self.view_name = None
+
+        # The string name of the Table under management by this viewer and its
+        # attached PerspectiveManager
+        self.table_name = None
 
         # Viewer configuration
         self.plugin = validate_plugin(plugin)
@@ -111,40 +137,47 @@ class PerspectiveViewer(PerspectiveTraitlets, object):
 
     @property
     def table(self):
-        '''Returns the ``perspective.Table`` under management by the viewer.'''
+        """Returns the ``perspective.Table`` under management by the viewer."""
         return self.manager.get_table(self.table_name)
 
-    @property
-    def view(self):
-        '''Returns the ``perspective.View`` currently shown by the viewer.
+    def load(self, data, **options):
+        """Given a ``perspective.Table``, a ``perspective.View``,
+        or data that can be handled by ``perspective.Table``, pass it to the
+        viewer.  Like `__init__`, load accepts a `perspective.Table`, a dataset,
+        or a schema. If running in client mode, `load` defers to the browser's
+        Perspective engine. This means that loading Python-only datasets,
+        especially ones that cannot be serialized into JSON, may cause some
+        issues.
 
-        This property changes every time the viewer configuration changes.'''
-        return self.manager.get_view(self.view_name)
+        ``load()`` resets the state of the viewer :
 
-    def load(self, table_or_data, **options):
-        '''Given a ``perspective.Table`` or data that can be handled by
-        ``perspective.Table``, pass it to the viewer.
+        * If a ``perspective.Table`` has already been  loaded, ``**options`` is
+          ignored as the options already set on the ``Table`` take precedence.
 
-        ``load()`` resets the state of the viewer.  If a ``perspective.Table``
-        is passed into ``table_or_data``, ``**options`` is ignored as the options
-        already set on the ``Table`` take precedence.  If data is passed in, a
-        ``perspective.Table`` is automatically created by this function, and the
-        options passed to ``**config`` are extended to the new Table.
+        * If a ``perspective.View`` is loaded, the options on the
+          ``perspective.Table`` linked to the view take precedence.
+
+        If data is passed in, a ``perspective.Table`` is automatically created
+        by this method, and the options passed to ``**config`` are extended to
+        the new Table.  If the widget already has a dataset, and the new data
+        has different columns to the old one, then the widget state (pivots,
+        sort, etc.) is cleared to prevent applying settings on columns that
+        don't exist.
 
         Args:
-            table_or_data (:obj:`Table`|:obj:`dict`|:obj:`list`|`pandas.DataFrame`): a
-                `perspective.Table` instance or a dataset to be displayed
-                in the viewer.
+            data (:obj:`Table`|:obj:`View`|:obj:`dict`|:obj:`list`|:obj:`pandas.DataFrame`|:obj:`bytes`|:obj:`str`): a
+                `perspective.Table` instance, a `perspective.View` instance, or
+                a dataset to be loaded in the viewer.
 
         Keyword Arguments:
             name (:obj:`str`): An optional name to reference the table by so it can
                 be accessed from the front-end. If not provided, a name will
                 be generated.
             index (:obj:`str`): A column name to be used as the primary key.
-                Ignored if a `Table` is supplied.
+                Ignored if a ``Table`` or ``View`` is supplied.
             limit (:obj:`int`): A upper limit on the number of rows in the Table.
-                Cannot be set at the same time as `index`, ignored if a `Table`
-                is passed in.
+                Cannot be set at the same time as `index`. Ignored if a
+                ``Table`` or ``View`` is supplied.
 
         Examples:
             >>> from perspective import Table, PerspectiveViewer
@@ -153,16 +186,22 @@ class PerspectiveViewer(PerspectiveTraitlets, object):
             >>> viewer = PerspectiveViewer()
             >>> viewer.load(tbl)
             >>> viewer.load(data, index="a") # viewer state is reset
-        '''
+            >>> viewer2 = PerspectiveViewer()
+            >>> viewer2.load(tbl.view())
+
+        """
         name = options.pop("name", str(random()))
-        if isinstance(table_or_data, Table):
-            table = table_or_data
+
+        if isinstance(data, Table):
+            table = data
+        elif isinstance(data, View):
+            raise PerspectiveError("Only `Table` or data can be loaded.")
         else:
-            table = Table(table_or_data, **options)
+            table = Table(data, **options)
 
         self.manager.host_table(name, table)
 
-        # Reset the viewer when `load()` is called again.
+        # Reset the viewer when `load()` is called multiple times.
         if self.table_name is not None:
             self.reset()
 
@@ -174,56 +213,56 @@ class PerspectiveViewer(PerspectiveTraitlets, object):
         self.table_name = name
 
     def update(self, data):
-        '''Update the table under management by the viewer with new data.
+        """Update the table under management by the viewer with new data.
         This function follows the semantics of `Table.update()`, and will be
         affected by whether an index is set on the underlying table.
 
         Args:
-            data (:obj:`dict`|:obj:`list`|:obj:`pandas.DataFrame`): the update data for the
-                table.
-        '''
+            data (:obj:`dict`|:obj:`list`|:obj:`pandas.DataFrame`): the
+            update data for the table.
+        """
         self.table.update(data)
 
     def clear(self):
-        '''Clears the rows of this viewer's ``Table``.'''
+        """Clears the rows of this viewer's ``Table``."""
         if self.table is not None:
             self.table.clear()
 
     def replace(self, data):
-        '''Replaces the rows of this viewer's `Table` with new data.
+        """Replaces the rows of this viewer's `Table` with new data.
 
         Args:
-            data (:obj:`dict`|:obj:`list`|:obj:`pandas.DataFrame`): new data to set into the
-                table - must conform to the table's schema.
-        '''
+            data (:obj:`dict`|:obj:`list`|:obj:`pandas.DataFrame`): new data
+            to set into the table - must conform to the table's schema.
+        """
         if self.table is not None:
             self.table.replace(data)
 
     def save(self):
-        '''Get the viewer's attribute as a dictionary, symmetric with `restore` so that a
-           viewer's configuration can be reproduced.'''
+        """Get the viewer's attribute as a dictionary, symmetric with `restore`
+        so that a viewer's configuration can be reproduced."""
         return {
-            'row_pivots': self.row_pivots,
-            'column_pivots': self.column_pivots,
-            'filters': self.filters,
-            'computed_columns': self.computed_columns,
-            'sort': self.sort,
-            'aggregates': self.aggregates,
-            'columns': self.columns,
-            'plugin': self.plugin,
+            attr: getattr(self, attr)
+            for attr in PerspectiveViewer.PERSISTENT_ATTRIBUTES
         }
 
     def restore(self, **kwargs):
-        '''Restore a given set of attributes, passed as kwargs (e.g. dictionary). Symmetric with `save` so that
-        a given viewer's configuration can be reproduced'''
+        """Restore a given set of attributes, passed as kwargs
+        (e.g. dictionary). Symmetric with `save` so that a given viewer's
+        configuration can be reproduced."""
         for k, v in six.iteritems(kwargs):
-            if k in ('row_pivots', 'column_pivots', 'filters', 'sort', 'aggregates', 'columns', 'computed_columns', 'plugin'):
+            if k in PerspectiveViewer.PERSISTENT_ATTRIBUTES:
                 setattr(self, k, v)
 
     def reset(self):
-        '''Resets the viewer's attributes and state, but does not delete or
+        """Resets the viewer's attributes and state, but does not delete or
         modify the underlying `Table`.
-        '''
+
+        Example:
+            widget = PerspectiveWidget(data, row_pivots=["date"], plugin=Plugin.XBAR)
+            widget.reset()
+            widget.plugin  #
+        """
         self.row_pivots = []
         self.column_pivots = []
         self.filters = []
@@ -232,55 +271,32 @@ class PerspectiveViewer(PerspectiveTraitlets, object):
         self.aggregates = {}
         self.columns = []
         self.plugin = "datagrid"
+        self.editable = False
 
     def delete(self, delete_table=True):
-        '''Delete the Viewer's data and clears its internal state. If
-        ``delete_table`` is True, the underlying `perspective.Table` and all
-        associated ``View`` objects will be deleted.
+        """Delete the Viewer's data and clears its internal state. If
+        ``delete_table`` is True, the underlying `perspective.Table` and the
+        internal `View` object will be deleted.
 
         Args:
             delete_table (:obj:`bool`) : whether the underlying `Table` will be
                 deleted. Defaults to True.
-        '''
-        if self.view:
-            self.view.delete()
-            self.view_name = None
-
-        for view in self.manager._views.values():
-            view.delete()
-
+        """
         if delete_table:
+            # Delete all created views on the widget's manager instance
+            for view in six.itervalues(self.manager._views):
+                view.delete()
+
+            # Reset view cache
+            self.manager._views = {}
+
+            # Delete table
             self.table.delete()
             self.manager._tables.pop(self.table_name)
             self.table_name = None
 
         self.reset()
 
-    def _new_view(self):
-        '''Create a new View, and assign its name to the viewer.  Do not call
-        this function - it will be called automatically when the state of the
-        viewer changes. There should only be one View associated with the
-        Viewer at any given time - when a new View is created, the old one
-        is destroyed.
-        '''
-        if not self.table_name:
-            return
-
-        name = str(random())
-        table = self.manager.get_table(self.table_name)
-        view = table.view(
-            row_pivots=self.row_pivots,
-            column_pivots=self.column_pivots,
-            columns=self.columns,
-            aggregates=self.aggregates,
-            sort=self.sort,
-            filter=self.filters,
-            computed_columns=self.computed_columns
-        )
-
-        self.manager.host_view(name, view)
-        self.view_name = name
-
     def __setattr__(self, name, value):
         """Override __setattr__ in order to allow Enums to be validated
         through Traitlets."""
